globals [target-color other-color background-color patches-in-block %-target-color-list]
patches-own [my-color]
breeds [frames]

to startup
    initialize
    set-colors
    chequer    
end

to initialize ;initialize variables
    ask frames [die]
    clear-plot
    set %-target-color-list []
end

;; We used color variables and not just colors so that you could change the color values
;; to suit you. For instance, perhaps you would want to change the green to cyan
to set-colors
    set target-color red
    set other-color green
    set background-color 44
end


;; This procedure starts the graphics window with a checkered display.  
;; This procedure's function is (a) to give the user a sense of the layout of the land 
;; (b) aesthetic. You can copy this basic code to your own models. 
to chequer
    ask patches 
    [
     ifelse  ( ( pxcor / 2  = int (pxcor / 2) ) and ( pycor / 2  = int (pycor / 2) ) ) or 
             ( ( pxcor / 2 != int (pxcor / 2) ) and ( pycor / 2 != int (pycor / 2) ) )
     
     [set pcolor target-color]
     [set pcolor other-color]
    ]
end

to setup 
    initialize  
    create-block
    create-frames
end

to create-block ;; creates in the graphics window a red block of all patches-in-block
    ask patch 0 0
      [set patches-in-block ( in-rectangle patches ((block-side - 1) / 2)  ((block-side - 1) / 2) ) ]
    ask patches-in-block [set pcolor background-color]
    ask patches with [pcolor != background-color] [set pcolor brown]
end

;; report the agentset within a box that has a width = (width/2 * 2) + 1
;; and a height = (height/2 * 2) + 1 and is centered on the calling agent
to-report in-rectangle [ patchset width/2 height/2 ]
  ;; this procedure does not work with a non-patch-agentset variable
  if not is-patch-agentset? patchset
  [ report nobody ]
  report patchset with
  [
    pxcor <= (pxcor-of myself + width/2) and pxcor >= (pxcor-of myself - width/2) and
    pycor <= (pycor-of myself + height/2) and pycor >= (pycor-of myself - height/2)
  ]
end

to create-frames ;; each individual patch in the block is framed, creating an overall effect  
                 ;; of a grid that helps distinguish individual patches in the block
    set-default-shape frames "frame"
    ask patches-in-block
    [
      sprout 1 
      [ set breed frames
        set color black ]  
    ]
end

to run
if target-color = 0 [stop]
    ask patches-in-block
    [
      ifelse random 100.0 < %-target-color
        [set pcolor target-color ] [set pcolor other-color]
    ]
    
    ;; We multiply by 100 in order to convert a probability, e.g., .23, into a percent -- 23.
    set %-target-color-list fput  ( 100 * ( count patches-in-block with [pcolor = target-color] ) / count patches-in-block ) %-target-color-list
    plot-color-distribution
    wait (5 - speed) / 5
end

to plot-color-distribution
    set-current-plot "color distribution"
    set-current-plot-pen "%-target-color"
    histogram-list %-target-color-list
end

; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2002 by Uri Wilensky.  Updated 2003.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo Stochastic Patchwork model.
; http://ccl.northwestern.edu/netlogo/models/StochasticPatchwork.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
329
10
665
346
10
10
16.0
1
10
0
0

CC-WINDOW
28
402
903
497
Command Center

SLIDER
71
62
289
95
block-side
block-side
1
17
3
2
1
NIL

SLIDER
67
357
296
390
%-target-color
%-target-color
0
100
50
1
1
%

BUTTON
71
12
134
59
NIL
setup
NIL
1
T
OBSERVER

BUTTON
224
10
287
59
NIL
run
T
1
T
OBSERVER

SLIDER
71
99
288
132
speed
speed
1
5
1
1
1
NIL

MONITOR
156
138
249
187
# target color
count patches-in-block with [pcolor = green]
3
1

PLOT
39
192
322
347
color distribution
% target color
occurrences
0.0
100.0
0.0
10.0
true
false
PENS
"%-target-color" 0.0010 1 -65536 true

@#$#@#$#@
WHAT IS IT?
-----------
This model is a part of the ProbLab curriculum. The ProbLab Curriculum is currently under development at the CCL. For more information about the ProbLab Curriculum please refer to http://ccl.northwestern.edu/ProbLab/.

The longer name of Stochastic Patchwork is Agent-Aggregate Probability Distribution. It is a simple model for thinking about Probability. Specifically, the user can study the relation between, on the one hand, individual agents' probability to have one of two values in the dichotomous variable color, and on the other hand, the aggregate or overall distribution of these values within a population of such patches. Moreover, by collecting these momentary aggregate probabilities and plotting them in a "growing" histogram, the probability distribution becomes a dynamic model of stochastic behavior. Thus, the model objectifies, problematizes, and elaborates the complementarity of agentive and aggregate perspectives on probability distribution.  


HOW IT WORKS
------------
At every iterated run, each patch "rolls a die" to decide whether it should be red or green.  This "die" works as follows: The patch chooses a random number between 0 and 100.  Next, the patch checks to see whether this number is smaller or larger than the value you set on the "%-target-color" slider.  If, for instance, the random number was 19 and the slider value was 25, then the patch will become red, but if -- say on the immediately consecutive run -- the random number happened to be 65 (and you have not changed the slider value from 25) then the patch will be green on this run.  At every run, the overall value of the population's "red-ness" is calculated as the number of red patches in the block divided by the total number of patches in the block. This calculated value is added to a growing list that thus grows in length from run to run.  The entire list is plotted as a histogram.

Over many runs, the histogram begins to form a bell-curve shape with your chosen %-target-color as a mean (and mode and median).


HOW TO USE IT
-------------
Buttons:
'Setup; - prepares the size of population according to the block-side slider value.
'Run'   - activates the procedures.  It is currently set to work "forever," that is, repeatedly until you press it again.

Sliders:
'block-side' - the larger the value you set here, the larger the size of your square population.  If the block size is X, then the block will be of dimensions (2X - 1) by (2X - 1).
'speed' - the higher the value you set here, the less the program will wait in-between iterations.  When you set the speed to its maximum value the program will run at its fastest.
'%-target-color' -  Use this slider to control the average probability that each patch will be red in each run.

Monitors:
'# target color' - shows at every run how many patches in the block are red. (Note that the plot window shows a histogram of the percent red, and not of the number of red patches.)

Set the sliders to the values of your choice, press Setup, and then press Run.


THINGS TO NOTICE
----------------
The smaller your block, the less possibilities there are for the decimal-fraction forms of the aggregate probability distributions.  That is why the histogram bars become sparse.  You can run the model with a small block size, say of slider-value 2, and then stop it after a while by pressing again on the Run button.  Now, in the command center, write "show %-target-color-list."  Notice how the low variety in decimal forms corresponds to the sparseness of bars in the histogram.


THINGS TO TRY
-------------
How long does it take for the histogram to start taking its typical bell shape?  How is this duration of time related, if at all, to the block size?

How is the shape of the curve related to the %-target-color value?  Choose a largish block size, then set the %-target-color slider to an extremely high value, such as 98%.  Let the model run until it begins to take some distinctive shape.  Now, stop the model by pressing the Run button again.  Then, without pressing Setup, re-set the %-target-color slider to, say, 70% and let it run again.  Repeat this several times for smaller and smaller percentages, ending with another extreme percentage, say 2%.  You will have a set of curves on your screen.  Are they all bell shaped?  -- If not, why not?

Ask a friend to set up the %-target-color value for you without your looking.  Now, cover the left-hand side of the screen (so that you see only the graphics window).  You can also do that by asking your friend to shift the entire NetLogo window to the left so that the left-hand side of the window is hidden.  Next, run the model (or have your friend run it and only then hide it).  Looking at the flashing colors on the population block, try to guess the percentage-red value.  Practice this, then switch roles with your friend.  Who is better at this task?  Discuss your strategies. (see 'Experts' in the EXTENDING section.) 


NETLOGO FEATURES
----------------
Look in the code at the procedure 'to create-block' and the reporter procedure 'in-rectangle' immediately below it. Note how adjustments need to be made in order to get the code to produce a block of patches with the 'block-side' the user sets. You may wish to familiarize yourself with what might seem at first confusing: Even though the patches create a grid-like structure, this grid is offset from the underlying coordinate system that gives patches their names. This real coordinate system runs through the patches centers and not along their perimeters. For instance, the location of "patch 0 0" is precisely that -- [0 0] -- because its center, that is, its pxcor and pycor, is on the origin. But in fact, the patch 0 0 extends from -.5 to +.5 along both the x and the y axes. 


EXTENDING THE MODEL
-------------------
A first jab at coding is to change the two focal colors from red and green to whatever you like.  In the Procedures tab, find the "set-colors" procedures and replace the existing colors with others.  You may wish to consult the NetLogo manual (in the "Help" drop-down menu) to find out which colors can be named and which have to be numbered.

On the interface, you can edit the Speed slider so as to allow for even slower runs.  For that, you would probably wish to understand the code in the "to run" procedure.

A more advanced intervention is to work with the plot window.  What other variables could it be interesting to monitor?  You can add a plotting of the average probability, or of the occurrence of a particular value.  You could create another window that graphs the aggregate probability as it changes over time. What do you expect that graph would look like - initially, later?  How, if at all, would the shape of that graph be related to the settings of the slider values?

Experts (continued from above section): Try extending the model so as to be able to improve your guesstimation strategies.  You can even create interactive procedures that will enable you to track you and your friend's progress.  Perhaps it is worthwhile to find ways to make your "guesstimations" more precise.  What could such techniques be?  How are they related to what scientists call "doing statistics"?  Finally, what can we learn about our perceptual mechanism from this exercise - its affordances and its limitations?  


CREDITS AND REFERENCES
----------------------
Thanks to Dor Abrahamson for his work on the design of this model and the ProbLab curriculum.

To refer to this model in academic publications, please use: Wilensky, U. (2002). NetLogo Stochastic Patchwork model. http://ccl.northwestern.edu/netlogo/models/StochasticPatchwork. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

circle
false
0
Circle -7566196 true true 35 35 230

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

bee
true
0
Polygon -256 true false 151 152 137 77 105 67 89 67 66 74 48 85 36 100 24 116 14 134 0 151 15 167 22 182 40 206 58 220 82 226 105 226 134 222
Polygon -16777216 true false 151 150 149 128 149 114 155 98 178 80 197 80 217 81 233 95 242 117 246 141 247 151 245 177 234 195 218 207 206 211 184 211 161 204 151 189 148 171
Polygon -7566196 true true 246 151 241 119 240 96 250 81 261 78 275 87 282 103 277 115 287 121 299 150 286 180 277 189 283 197 281 210 270 222 256 222 243 212 242 192
Polygon -16777216 true false 115 70 129 74 128 223 114 224
Polygon -16777216 true false 89 67 74 71 74 224 89 225 89 67
Polygon -16777216 true false 43 91 31 106 31 195 45 211
Line -1 false 200 144 213 70
Line -1 false 213 70 213 45
Line -1 false 214 45 203 26
Line -1 false 204 26 185 22
Line -1 false 185 22 170 25
Line -1 false 169 26 159 37
Line -1 false 159 37 156 55
Line -1 false 157 55 199 143
Line -1 false 200 141 162 227
Line -1 false 162 227 163 241
Line -1 false 163 241 171 249
Line -1 false 171 249 190 254
Line -1 false 192 253 203 248
Line -1 false 205 249 218 235
Line -1 false 218 235 200 144

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 46 130
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150
Polygon -7566196 true true 270 14
Polygon -7566196 true true 276 21

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

frame
false
14
Rectangle -16777216 true true -8 2 18 298
Rectangle -16777216 true true 1 0 300 16
Rectangle -16777216 true true 283 2 299 300
Rectangle -16777216 true true 1 285 300 299

@#$#@#$#@
NetLogo 1.2beta4 (Rev C)
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
