breeds [ reactants products ]    ;; reactants are green, products are red
turtles-own [ reacting? ]       ;; used so greens can only participate in one reaction at a time

to setup
  ca
  set-default-shape reactants "molecule1"
  set-default-shape products "molecule2"
  create-custom-reactants number
    [ set color green
      set xcor screen-edge-x - random screen-size-x
      set ycor screen-edge-y - random screen-size-y
      set reacting? false ]
  setup-plot
  do-plotting
end

to go
  every ln (10 / speed)
    [ ask turtles
        [ rt (random 10 - random 10)  ;; wander around randomly
          fd 1 ]
      ask reactants
        [ react-forward ]
      ask products
        [ react-backward ]
      do-plotting ]
end

to react-forward
  locals [partner]  ;; the turtle we're going to try and react with
  if (any other-reactants-here) and (random 1000 < (Kb * 2))  ;; multiply Kb by 2 because 2 molecules are involved
    [ set reacting? true  ;; prevent anyone else from trying to react with us
      set partner (random-one-of other-reactants-here)
      if not reacting?-of partner
        [ set reacting?-of partner true  ;; prevent anyone else from trying to react with our partner
          set breed-of partner products
          set reacting?-of partner false
          set color-of partner red
          die ]
      set reacting? false ]
end

to react-backward
  if ((random 1000) < Ku)
    [ set breed reactants    ;; change back to reactant
      set color green
      hatch 1 [ ] ]         ;; then split into two reactants
end

;;; plotting procedures

to setup-plot      ;; the code below sets up the plot
  set-current-plot "Concentration"
  set-plot-y-range 0 number
end

to do-plotting      ;; plot concentrations
  set-current-plot "Concentration"
  set-current-plot-pen "reactants"
  plot count reactants
  set-current-plot-pen "products"
  plot count products
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Simple Kinetics 1 model.
; http://ccl.northwestern.edu/netlogo/models/SimpleKinetics1.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
274
10
624
360
12
12
14.0
1
10
0
0

CC-WINDOW
274
363
625
498
Command Center

SLIDER
12
89
261
122
Kb
Kb
0.0
100.0
20.0
1.0
1
NIL

SLIDER
12
124
261
157
Ku
Ku
0.0
100.0
7.0
1.0
1
NIL

BUTTON
12
10
79
43
setup
setup
NIL
1
T
OBSERVER

BUTTON
82
10
144
43
go
go
T
1
T
OBSERVER

PLOT
3
183
271
428
Concentration
time
# molecules
0.0
35.0
0.0
300.0
true
true
PENS
"reactants" 1.0 0 -11352576 true
"products" 1.0 0 -65536 true

SLIDER
12
54
261
87
number
number
1
1000
300
1
1
NIL

SLIDER
150
10
261
43
speed
speed
1.0
10.0
9.2
0.1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------

This model demonstrates the kinetics of a simple reversible reaction.  It demonstrates numerically that the appliction of the Principle of Stationary Concentrations is valid in this case.

In this model there are two kinds of molecules, green and red.  Green turtles turn into red turtles bimolecularly whereas red turtles turn back into pairs of green turtles monomolecularly.  You can control the rate at which this changes occur with sliders Kb and Ku.

The reaction here is a reversible reaction of the type:

|                         Kb
|        A + A <=======> B
|                         Ku

An example of such a reaction would be dimerization of acetic acid:

|                                  Kb
|        2 H C-COOH <=======> H C-COOH~...~HOOC-C H
|       3           Ku     3                 3 


This reaction is an example of a complex reaction which consists of two elementary reactions.  The forward bimolecular reaction 
                
|                        Kb
|        A + A --------> B 


is characterized by the constant Kb and the reverse unimolecular reaction

|             Ku
|        B ---------> A + A


is chracterized by the constant Ku.
                                 
The system of ordinary differential equations (ODE) that describes the concentrations of A and B is given below:


|        dA           2
|        -- = -2Kb * A  + 2Ku * B        (1)
|        dt            

|        dB         2
|        -- = Kb * A  - Ku * B           (2)
|        dt


The usual initial conditions are A(0) = Ao and B(0) = 0.  While it is possible to solve this system of ODE analytically, chemists usually apply the Principle of Stationary Concentrations when they investigate the kinetics of reactions of this type.  The Principle says that one can assume that the concentrations of the species stop changing from some point on after the system reaches equilibrium.  If concentrations are stationary, the derivatives

|        dA               dB
|        --  and  --
|        dt               dt

are zero.  Hence one can replace the system of ODE above with the systemn of algebraic equations below:
|                    2       
|        0 = -2Kb * A*   + 2Ku * B*              (1')
|
|                  2     
|        0 = Kb * A*  - Ku * B*                  (2')

where concentrations marked with * are stationary concentrations.  The second equation (2') is linearly dependent on the first equation (1').  Luckily we also have another equation coming from the law of the conservation of mass:

|        A* + 2 * B* = Ao                        (3)

From equation (2') we can express B* in terms of A* :

|                  Kb    2
|        B*  = --  A*                    (4)
|                  Ku    

We can now plug in expression (4) into (3) and then we will have a quadratic equation in terms of A*:

|                  Kb   2
|        A*  + -- A*  = Ao               (5)
|          Ku
        
whose solution is:

|             _____________
|             |
|             |             Kb     
|             |  1 + 4 * --  - 1
|            \|          Ku
|           --------------------         (6)
|                  Kb
|                         2 * --
|                                 Ku

One can now find the stationary concentration of B using equation (4).


HOW TO USE IT
--------------

Choose the values of Ku and Kb with appropriate sliders:
- Kb controls the rate of the forward reaction by which two green turtles turn bimolecularly into a single red turtle. 
- Ku controls the rate of the reverse reaction, by which a red turtle turns unimolecularly into two green turtles. 

Having chosen appropriate values of the constants, press SETUP to clear the screen and create an initial number of green turtles.  Note: we do not create red turtles initially, although this could be done in principle.

Press GO to start the simulation.  


THINGS TO NOTICE
----------------

You will see turtles wandering around the screen and  changing color.  Pay more attention to the plot of the concentrations.  Do the plots soon reach stationary concentrations? 


THINGS TO TRY
-------------

How do the stationary concentrations depend on the values of Kb and Ku?   You can change Ku and Kb while the model is running.   See if you can predict what the stationary concentrations will be with various combinations of Kb and Ku. 


EXTENDING THE MODEL
------------------

Try to implement the following reaction:

|                 Kb         K2
|        A + A <======> B -------> C
|                 Ku

This reaction underlines a vast number of microbiological processes (e.g. fermentation).  You can read about its kinetics in any book on Biochemistry.  Look up the so-called Michaeles-Menten equation.  Does it check numerically? 

Try to implement the following reaction:

|                 Kb         K2
|        A + B <======> C -------> D
|                 Ku


NETLOGO FEATURES
------------------
It is a little tricky to ensure that a reactant never participates in two reactions simultaneously.  The turtle variable REACTING? is used to control this.  In the future, a primitive called GRAB may be added to NetLogo; then REACTING? won't be needed.


CREDITS AND REFERENCES
----------------------
Thanks to Mike Stieff for his work on this model.

To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Simple Kinetics 1 model. http://ccl.northwestern.edu/netlogo/models/SimpleKinetics1. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

molecule3
true
0
Circle -7566196 true true 92 92 118
Circle -1 true false 120 32 60
Circle -1 true false 58 183 60
Circle -1 true false 185 183 60

molecule1
true
0
Circle -7566196 true true 80 80 142
Circle -1 true false 52 195 61

molecule2
true
0
Circle -7566196 true true 78 79 146
Circle -1 true false 40 41 69
Circle -1 true false 194 42 71

@#$#@#$#@
NetLogo 1.1 (Rev C)
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
