;; Setup Procedures
to setup
  ca
  cct bluemols
  [ set color blue
    randomize
  ]
  cct yellowmols
  [ set color yellow
    randomize
  ]
  ;; adjust the y-range of the plot to fit the number of molecules
  set-plot-y-range 0 max (list yellowmols bluemols)
end

to randomize
  set shape "circle"
  setxy random screen-size-x random screen-size-y
  rt random 360
end


;; Runtime Procedures

;; In GO, turtles move according to a number of different checks and
;;  procedures.  Note that all turtles must have executed the procedure
;;  CHECK-FOR-SPACE before any turtle can continue with CHECK-FOR-REACTION
;;  because they are broken up into two seperate ask blocks.  This ensures
;;  that every turtle will be on its own patch before any turtles checks for
;;  a reaction.
to go
  update-plot
  ask turtles
  [ fd 1 
    wiggle
    check-for-space
  ]
  ask turtles
  [ check-for-reaction ]
end

;; In WIGGLE, turtles are given a slight random twist to their heading.
to wiggle
  rt random 2
  lt random 2
end

;; In CHECK-FOR-SPACE, each turtle moves forward until it is alone in its
;;  own patch.  Note that the procedure calls itself if the check is true.
;;  This technique is called recursion.  CHECK-FOR-SPACE is called in GO.
to check-for-space
  if (count turtles-here > 1) 
  [ fd 1 check-for-space ]
end

;; In CHECK-FOR-REACTION, every turtle checks for a turtle it can react with within
;;  a radius which approximates the eight patches surrounding it.  Blue turtles can
;;  react with yellow turtles, and brown turtles can react with green turtles.
to check-for-reaction
  if color = blue and any turtles in-radius 1 with [color = yellow]
  [ react-forward random-one-of turtles in-radius 1 with [color = yellow] ]
  if color = brown and any turtles in-radius 1 with [color = green]
  [ react-reverse random-one-of turtles in-radius 1 with [color = green] ]
end

;; In REACT-FORWARD and REACT-REVERSE, the turtle changes the its color and the color
;;  of the turtle they are reacting with, sets a random heading, and jumps away to
;;  avoid further interaction.  They are the same except for their colors.
to react-forward [t]
  set color-of t brown
  set color green
  rt random 360
  jump 2
end

to react-reverse [t]
  set color-of t blue
  set color yellow
  rt random 360
  jump 2
end

;; In UPDATE-PLOT, each pen plots its corresponding turtle count.
to update-plot
  set-current-plot-pen "Yellows"
    plot count turtles with [color = yellow]
  set-current-plot-pen "Blues"          
    plot count turtles with [color = blue]
  set-current-plot-pen "Browns"
    plot count turtles with [color =  brown]
  set-current-plot-pen "Greens"
    plot count turtles with [color = green]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Chemical Equilibrium model.
; http://ccl.northwestern.edu/netlogo/models/ChemicalEquilibrium.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
289
10
658
379
20
20
9.0
1
10
0
0

CC-WINDOW
9
391
277
499
Command Center

BUTTON
57
10
134
43
setup
setup
NIL
1
T
OBSERVER

BUTTON
152
10
228
43
go
go
T
1
T
OBSERVER

SLIDER
9
52
143
85
yellowmols
yellowmols
0
500
100
1
1
NIL

SLIDER
143
52
277
85
bluemols
bluemols
0
500
200
1
1
NIL

MONITOR
143
85
221
134
blues
count turtles with [color = blue]
3
1

MONITOR
65
85
143
134
yellows
count turtles with [color = yellow]
3
1

MONITOR
143
134
221
183
greens
count turtles with [color = green]
3
1

MONITOR
65
134
143
183
browns
count turtles with [color = brown]
3
1

PLOT
9
193
277
389
Concentrations
time
molecules
0.0
100.0
0.0
200.0
true
true
PENS
"Yellows" 1.0 0 -256 true
"Blues" 1.0 0 -16776961 true
"Greens" 1.0 0 -11352576 true
"Browns" 1.0 0 -6524078 true

@#$#@#$#@
WHAT IS IT?
-----------
This project shows how a simple chemical system comes to different equilibrium states depending on the concentrations of the initial reactants.  Equilibrium is the term we use to describe a system in which there are no macroscopic changes.  This means that the system "looks" like nothing is happening.  In fact, in all chemical systems microscopic processes continue but in a balance that yields no changes at the macroscopic level.  This model simulates two simple reactions of four molecules.   

The reactions can be written A + B yields C + D.  And at the same time, of course, C + D yields A + B.

A classic real-life example that would illustrate such reactions is the reactions of carbon monoxide with nitrous dioxide to produce carbon dioxide and nitrous monoxide.  The reverse is also possible.  All the reactants are gases.  We could watch such an equilibrium system because NO2 is a reddish colored gas which is visible. However, the combining of nitrous dioxide (NO2) with carbon monoxide (CO) results in the colorless products nitrous monoxide (NO) and carbon dioxide (CO2), and so the system loses its reddish color.  And yet, not all the color is lost. Ultimately the system comes to equilibrium with some of the "reactants" and some of the "products" present.

How much "reactant" and "product" a system ends up with depends on a number of factors.  The inherent kinetics of the reaction are of vital concern: For instance, some reactions tend to go in a particular direction because energy is released in that direction.  A system's equilibrium is also affected by the concentrations of the reactants -- this is modeled here -- and by the system's temperature.


HOW TO USE IT
-------------
As stated above, this model simulates a chemical system of four different molecules.  They are represented on the graphics screen as turtles of four different colors.  In this simulation, yellow molecules react with blue molecules to produce brown molecules and green molecules.  

The model is setup by first adjusting the YELLOWMOLS and BLUEMOLS sliders and pushing the SETUP button.  YELLOWMOLS sets how many yellow molecules the simulation starts with, while BLUEMOLS sets how many blue molecules the simulation starts with.


The GO button sets the simulation in motion.  Molecules move randomly and react with each other, changing color to represent rearrangement of atoms into different molecular structures.  The system soon comes into equilibrium.

Four monitors show how many of each kind of molecule are present in the system.  There is also a plot which plots the number of each kind of molecule present versus time.



THINGS TO NOTICE
----------------
Notice that the number of product molecules is limited by the smallest amount of reactant product.  Notice that there are always the same number of reactant products since they are formed in a one-to-one correspondence with each other. 


THINGS TO TRY
-------------
How do different amounts of the two reactants affect the final equilibrium.  Are absolute amounts important, is it the difference between the amounts, or is it a ratio of the two reactants that matters?  

Try setting the YELLOWMOLS slider to 400 and the BLUEMOLS slider to 20, 40, 100, 200, and 400 in five successive simulations.  What sort of equilibrium state do you predict in each case?  Are certain ratios predictable?


EXTENDING THE MODEL
-------------------
What if the forward and reverse reaction rates were the variables controlled instead of initial concentrations.  You could compare such a simulation with the one in this model and see if concentration and reaction rates act independently of each other, as measured by the final equilibrium state.  

You could also extend the program by allowing the user to introduce new molecules into the simulation while it is running.  How would the addition of fifty blue molecules affect a system that was already at equilibrium?


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2002). NetLogo Chemical Equilibrium model. http://ccl.northwestern.edu/netlogo/models/ChemicalEquilibrium. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

arrow
true
0
Polygon -7566196 true true 150 300 300 150 195 150 195 7 105 7 105 150 105 150 0 150

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

spacecraft
true
0
Polygon -7566196 true true 150 300 120 165 45 45 75 60 150 120 225 60 255 45 180 165

thin-arrow
true
0
Polygon -7566196 true true 150 300 300 150 180 150 180 4 120 4 120 150 120 150 0 150

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

circle
true
0
Circle -7566196 true true 35 35 230

@#$#@#$#@
NetLogo 1.2pre3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
