breeds [ neutrons rods ]
globals
[ power
  old-power      ; list of last 5 power values - used to compute average power
  average-power
  power-change
]

;;;;;;;;;;;;;;;;;;;;;;;;
;;; Setup Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;
to setup 
  ca
  set power 0
  set old-power [0 0 0 0 0]
  ask patches
  [ build-reactor
    setup-nuclear-fuel
  ]
  release-neutron
  setup-plot
  plot-power
end

to build-reactor ;; Patch Procedure
  if ((abs pxcor) = screen-edge-x) or ((abs pycor) = screen-edge-y)
  [ set pcolor gray ]
end

to setup-nuclear-fuel ;; Patch Procedure
  if (pcolor = black) and (percent-fuel > (random 100))
  [ set pcolor red ] 
end

to setup-plot
  set-current-plot "Power"
  set-plot-y-range 0 (3 * power-rated)
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Run Time Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;

to react
  if (slow?) [ wait 0.2 ]
  set power 0
  ask neutrons
  [ fd  1
    if (pcolor = gray or any rods-here)
    [ die ]
    if (pcolor = red)
    [ fission ]
  ]
  calculate-power
  plot-power
end

to calculate-power
  ;;average past five most recent power readings
  set power-change (power - first old-power)
  set old-power fput power but-last old-power
  set average-power mean old-power
end

to auto-react ;; Button
  ifelse (power-change >= 0)
  [ if (power-rated <= power)
    [ add-rods ]
  ]
  [ if (power < power-rated)
    [ remove-rods ]
  ]
  react
  if not any neutrons
  [ stop ]
end

to fission ;; Neutron Procedure
  locals [ gain ]
  rt (random 360)
  if (spend-fuel?)
  [ stamp brown ]
  set gain (1 / count neutrons-here)
  set power power + gain
  hatch ((2 + random 2) * gain)
  [ rt random 360 ]
end

to release-neutron ;; Button
  locals [ success ]
  set success false
  while [not success]
  [
    create-custom-neutrons 1
    [ set breed neutrons
      set color yellow
      rt (random 360)
      setxy ((random (screen-size-x - 2)) - screen-edge-x)
            ((random (screen-size-y - 2)) - screen-edge-y)
      ifelse any rods-here
      [ die ]
      [ set success true ]
    ]
  ]
end

to add-rods ;; Button
  create-custom-rods number
  [ set color cyan
    setxy ((random (screen-size-x - 2)) - screen-edge-x + 1)
          ((random (screen-size-y - 2)) - screen-edge-y + 1)
  ]
end

to remove-rods ;; Button
  ifelse (number >= (count rods))
  [ ask rods [ die ] ]
  [ repeat number
    [ ask random-one-of rods [ die ] ]
  ]
end

to plot-power
  set-current-plot-pen "power-rated" 
  plot power-rated 
  set-current-plot-pen "avg-power" 
  plot average-power
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Reactor Top Down model.
; http://ccl.northwestern.edu/netlogo/models/ReactorTopDown.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
344
10
708
374
45
45
4.0
0
10
0
0

CC-WINDOW
344
383
708
487
Command Center

SLIDER
9
10
223
43
power-rated
power-rated
0.0
100.0
100.0
1.0
1
NIL

BUTTON
9
119
107
152
Setup
setup
NIL
1
T
OBSERVER

BUTTON
225
119
334
152
Manual
react\nif not any neutrons\n[ stop ]
T
1
T
OBSERVER

BUTTON
107
119
225
152
Automatic
auto-react
T
1
T
OBSERVER

MONITOR
232
10
334
59
Power
power
3
1

MONITOR
232
60
334
109
Power change
power-change
3
1

PLOT
9
248
334
487
Power
time
power
0.0
250.0
0.0
60.0
true
true
PENS
"power-rated" 1.0 0 -16776961 true
"avg-power" 1.0 0 -65536 true

SWITCH
9
76
133
109
spend-fuel?
spend-fuel?
1
1
-1000

SWITCH
133
76
223
109
slow?
slow?
0
1
-1000

BUTTON
9
205
82
238
Add Rods
add-rods
NIL
1
T
OBSERVER

BUTTON
82
205
192
238
Remove Rods
remove-rods
NIL
1
T
OBSERVER

SLIDER
9
43
223
76
percent-fuel
percent-fuel
0.0
100.0
10.0
1.0
1
%

SLIDER
192
205
334
238
number
number
1
500
200
1
1
rods

BUTTON
94
162
234
195
Release Neutron
release-neutron
NIL
1
T
OBSERVER

@#$#@#$#@
WHAT IS IT?
-----------
This project simulates a nuclear fission reaction in a nuclear power plant. In a fission reaction, free neutrons hit uranium atoms, causing each uranium atom to generate 2 or 3 neutrons and a unit of energy. The uranium atom itself splits into two smaller atoms. The newly generated neutrons, together with the neutron that caused the reaction, keep moving and continue to hit more uranium atoms, which release more neutrons, etc. This is the chain reaction that happens inside an atomic bomb.

Most nuclear energy is used for peaceful purpose, however. Generated in a nuclear power plant, nuclear fission goes on in a much more controlled fashion. The casing of the reactor is made of thick plates of lead and concreate to absorb the neutrons and prevent them from escaping. The reaction is controlled by rods made of graphite, hafnium, cadmium, or boron which also absorb the free neutrons. The rods are added and removed from the reaction chamber in order to control the speed of the reaction. When control rods are added, fewer atoms of uranium are smashed, which in turn limits how much energy is released.

This model simulates the process of a nuclear fission reaction inside such a plant. The reactor casing seals the core, with control rods to help control the speed of the reaction. The reactor has a built-in automatic controller, and has a set of manual controls as well.

HOW TO USE IT
-------------
-- The POWER-RATED slider sets the rated power (the amount of power production that you want to hold the reactor to), which is used for automatic control.
-- The PERCENT-FUEL slider control the percent of the reactor that is filled with fuel (red patches).
-- The SPEND-FUEL switch controls whether or not the fuel is used up when a neutron hits it. (turns brown)
-- The ADD-RODS and REMOVE-RODS buttons work with the NUMBER slider to add or remove that number of rods (cyan turtles) from the reactor.
-- The RELEASE NEUTRON button introduces a neutron (yellow turtle) into the reactor. It can be used at anytime before or during the reaction.
-- The SLOW switch controls whether or not to slow the model run.

Press the SETUP button to set up the reactor with no control rods and a single neutron.

When MANUAL is on, use the ADD-RODS and REMOVE-RODS buttons to change the number of rods in the reactor, and thus control the speed of the reaction and the amount of power generated. If they are too few rods, too few neutrons are absorbed and the reactor will generate too much power. If they are too many, all free neutrons will be absorbed, and no power will be generated.

When AUTO-REACT is on, the reactor will add and remove rods as needed (in NUMBER increments) to keep power production under control.

THE POWER monitor shows the current power output.

The POWER-CHANGE monitor shows the change in power over the last clock tick.

Look at the POWER-PLOT plot to see the power curve.

THINGS TO NOTICE
----------------
Observe the fluctuation of the power curve. The blue line is the rated-power line and the red line is the actually generated power line.

THINGS TO TRY
-------------
There are two setups, one for manual control, one for automatic control.

The manual controls are very simple.  The ADD-RODS and REMOVE-RODS buttons control how many control rods are in the reactor.

The automatic controller is basically an ON and OFF controller. It is based on 1) comparison of power generated and power rated and 2) the power change. The mechanism is described in the following table:

|                     Power > Power Rated   Power < Power Rated
|                     -------------------   -------------------
|  Power Change > 0:       Add Rods            Do Nothing
|  
|  Power Change < 0:      Do nothing           Remove Rods

The controller should be fine-tuned.

* In Manual Mode:
Use the NUMBER slider to adjust the number to add or remove according to the information given through the two monitors and the plot. You want the power curve to be as flat as possible. The height of the curve should be close to the horizontal line representing the power-rate.

* In Automatic Mode:
Fine tune the controller by changing the mechanisms to get a flater power curve and minimize the fluctuation.

The SPEND-FUEL? switch sets whether or not the fuel is used up after releasing neutrons.  If it's on then the fuel in the reactor will quickly get used up and the model will end.  Because of the short run and the need for new fuel in order to continue the reaction, with spend-fuel? on it's best to have a lot of fuel in the reactor and a low power-rating.  If it's off then the fuel will never get used up and will react every time a neutron hits it and the model will probably never stop.  Because of the reactivity and longevity of this setting, it's best to have only a little fuel (otherwise it will react out of control) and a high power-rating, so the controller doesn't have to fill the core with control rods the entire time in order to hold to the rating.

EXTENDING THE MODEL
-------------------
The automatic controller used is just an on and off controller. Try using  Proportional, Integral and Differential (PID) controllers to make the reaction more smooth. You could also use some ideas of fuzzy logic (e.g. fuzzy membership) to build a fuzzy controller.

Absorbing free neutrons to control the reaction process is only one of the control mechanisms and is not the most important one. The important factor in controlling the reaction is to keep the speed of a neutron lower than a certain level (to stimulate fission reaction). The speeds of free neutrons depend on the density of the steam. Higher density yields lower speed and vice-versa. This is by nature a negative feedback control mechanism. It works as follows. If generated power is high, then the steam temperature is high and the density of the steam is low, thus the speed of free neutrons is high and is less likely to stimulate fission reaction. Hence a lower generated power. Try to incorporate this process into the automatic controller.

Neither setting of SPEND-FUEL? is completely realistic, with it on the fuel is used up too fast and there's hardly a reaction, with it off there is too much reacting and too many rods are needed to control the reaction.
Two addition ways to deal with fuel are listed below, can you think of more?
- have there be a certain ammount of fuel at each red patch, then it won't get used up right away, but won't go on forever either.
- limit how often a patch can react, that way, with spend-fuel? off the neutrons won't react out of control in small areas.

RELATED MODELS
--------------
Together with the Nuclear Reactor (cross-section) model you can get a decent representation of how a Nuclear Reactor might work in three dimensions.  These two models are variations of one another based off of the Fission model from StarLogoT.

NETLOGO FEATURES
-----------------
More so than most other NetLogo models, the plot of 'Nuclear Reactor' plays an active role in the simulation. When using the manual controller, it is quite helpful to watch the power curve, to tell when things are getting out of hand.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Reactor Top Down model. http://ccl.northwestern.edu/netlogo/models/ReactorTopDown. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

circle
true
0
Circle -7566196 true true 35 35 230

@#$#@#$#@
NetLogo 1.1 (Rev C)
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
