globals [ y-upper-bound counters ] 
breeds [ pegs queue-balls falling-balls piling-balls piled-balls ]
patches-own [ shade ]

to setup
  ca
  ; need y-upper-bound to be even
  set y-upper-bound screen-edge-y - 2 + (screen-edge-y mod 2)
  setup-balls  
  setup-pegs
end

to go
  locals [stop-running?]
  every slow-down
  [ set stop-running? false
    move-queue
    ask counters
      [ if any falling-balls-at 0 1 
          [ ask falling-balls-at 0 1 [ set breed piling-balls ]
            set plabel plabel + 1 
          ]
        ; stop running when the pile of balls reaches the green patches
        if any piled-balls-at 0 -1 [ set stop-running? true ]   
      ]
    ifelse pile-up? ; if not pile-up, just kill balls at bucket
       [ pile-up-balls ]
       [ ask piling-balls [ die ] ] 
    if stop-running? or (not any piling-balls and not any falling-balls)
    [
      while [any piling-balls]
      [ every slow-down
        [ pile-up-balls ]
      ]
      stop
    ]
    set-peg-angle
    do-trials
  ]
end

to setup-pegs
  locals [bucket-xcor]
  set-default-shape pegs "default"
  ask patches with [valid-pycor? pycor and valid-pxcor? pxcor pycor]
    [ sprout 1 [ set color yellow set breed pegs ] ] 
  set-peg-angle
  set counters patches with [valid-counter-patch? pycor]
  ask counters
    [ set plabel 0
      set pcolor scale-color green 40 50 0
      ; set the serial labels at the bottom of the graphics screen
      set plabel-of (patch pxcor (-1 * screen-edge-y)) round ((number-of-rows + pxcor) / 2)
    ]
end

to setup-balls
  set-default-shape turtles "circle"
  create-custom-queue-balls number-of-balls 
    [ set color red
      set heading 180
      set xcor 0
      set ycor screen-edge-y - 1 + screen-edge-y mod 2
    ]
end

;; moves balls one by one from the top of the screen
to move-queue
  if any queue-balls
    [ ask one-of queue-balls [ set breed falling-balls ] ]
  ask falling-balls
    [ fd 1
      shade-patch
      fd 1
      shade-patch
    ]
end

to do-trials
  ask falling-balls
    [ ifelse random 100 < chance-of-bouncing-right
        [ lt 90 ] ; actually turning right
        [ rt 90 ] ; actually turning left
      fd 1
      shade-patch
      set heading 180
    ]
end

;; pile up balls in columns
to pile-up-balls
  ask piling-balls 
    [ pu
      repeat 2
      [
        ifelse ycor != -1 * screen-edge-y and not any piled-balls-at 0 -1
          ; have to make sure piling-balls aren't on top of each other
          [ if not any piling-balls-at 0 -1 [ fd 1 ] ]
          [ set breed piled-balls ]
      ]
    ]
end

to set-peg-angle
  ask pegs
    [ ifelse chance-of-bouncing-right >= 50
        [ set heading 360 - (chance-of-bouncing-right - 50) * 0.9 ]
        [ set heading 45 - (chance-of-bouncing-right * 0.9) ]
    ]
end

;; shades the path of the balls, but only to a certain darkness
;; you can adjust the darkness by changing the value of darkness
to shade-patch 
  locals [darkness]
  set darkness 1.5
  if shade-path?
    [ ask patch-at 0 0
        [ if shade < number-of-balls / darkness
          [ set shade shade + 2
            set pcolor scale-color blue shade number-of-balls 0 
          ]
        ]
    ]
end

;;; predicate like reporters that help with graphic drawing
;;; valid coordinates depend on the number of rows in the triangle
;; a valid y coordinate is even coordinate, below the y-upper-bound
to-report valid-pycor? [y]
  report y >= y-upper-bound - (2 * number-of-rows) and y <= y-upper-bound and y mod 2 = 0
end

;; a valid x coordinate depends on a patches y coordinate
;; vertically, patches with valid x coordinates alter from having
;; odd to even pxcor
to-report valid-pxcor? [x y]
  if x <= pxcor-bounds y and x >= -1 * pxcor-bounds y
    [ if (pxcor-bounds y) mod 2 = 0 and x mod 2 = 1 and x != 0
        [ report true ]
      if (pxcor-bounds pycor) mod 2 = 1 and pxcor mod 2 = 0
        [ report true ]
    ]
  report false
end

;; below last row and inline with a peg
to-report valid-counter-patch? [y]
  report y = y-upper-bound - 2 * (number-of-rows + 1) and (any pegs-at 1 2 or any pegs-at -1 2)  
end

;; helps restrict pxcor depending on the size of y.
;; lower y ~ larger pxcor range
to-report pxcor-bounds [y]
  report (y-upper-bound - y) / 2
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2002 by Uri Wilensky.  Updated 2002.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo Galton Box model.
; http://ccl.northwestern.edu/netlogo/models/GaltonBox.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
231
10
546
559
17
30
9.0
1
9
0
0

CC-WINDOW
14
317
206
526
Command Center

SLIDER
12
95
208
128
number-of-rows
number-of-rows
1
15
12
1
1
NIL

BUTTON
29
10
106
43
setup
setup
NIL
1
T
OBSERVER

SLIDER
12
138
208
171
number-of-balls
number-of-balls
0
200
75
1
1
NIL

SLIDER
12
180
208
213
chance-of-bouncing-right
chance-of-bouncing-right
0
100
50
1
1
%

BUTTON
118
10
194
43
go
go
T
1
T
OBSERVER

SWITCH
44
222
174
255
shade-path?
shade-path?
1
1
-1000

SLIDER
12
53
208
86
slow-down
slow-down
0
2
0.0
0.1
1
NIL

SWITCH
58
264
163
297
pile-up?
pile-up?
0
1
-1000

@#$#@#$#@
WHAT IS IT?
-----------
A Galton Box is a triangular board that contains several rows of staggered but equally spaced pegs.  Balls are dropped from the top, bounce off the pegs and stack up at the bottom of the triangle.  In this model, the user is able to adjust the probability of the balls bouncing right or left when it hits a peg.  

The model enables you to observe how nature produces the binomial coefficients from Pascal's Triangle and their relation to a Gaussian bell-shaped normal curve. The model can also simulate coin tossing experiments with biased coins which result in skewed distributions

There are many applications for the concepts encompassed in a Galton Box.  People employed in a wide variety of fields use binomial and normal distributions to make precise calculations about the likelihood of events or groups of events occurring.
 

HOW TO USE IT                    
-------------
Click the SETUP to set up the rows of the triangle, the number of balls, and other parameters.  Click to GO button to begin the simulation.

The SLOW-DOWN slider controls how fast the model is running.  The larger the value, the slower the model runs.  

The SHADE-PATH? button gives the user the option to have the balls chart their path as they fall down the triangle.  The more balls that cover a particular path, the darker the path will become.  This option provides a graphical tool that helps the user define the most used path.

The PILE-UP? button controls if the balls create piles or simply disappear when they reach the bottom of the triangle.  If PILE-UP? is on and the pile of balls reaches the bottom of the triangle, the model will stop.  Note: If you are running a simulation with a  large number of balls you might want to turn PILE-UP? off.


THINGS TO NOTICE
----------------
With a small number of balls it is hard to notice any patterns as the balls pile up.

As you increase the number of balls, clear patterns and distributions start to form.  By adjusting the CHANCE-OF-BOUNCING-RIGHT you can see how different factors can change the distribution of balls.  What types of distributions form when the CHANCE-OF-BOUNCING-RIGHT is set at 20, 50, or 100?

This model is a good example of an independent trials process.  Each ball has a probability of turning one way, and its decision is unrelated to that of any of the other balls.  The large number of rows the balls must travel affects the amount of variation present in a run of the model.


THINGS TO TRY
-------------
Change the NUMBER-OF-BALLS and NUMBER-OF-ROWS sliders.  How does varying numbers alter how balls stack up?

Change the CHANCE-OF-BOUNCING-RIGHT slider as balls have begun to fall.  What kinds of ball distributions can you produce?

Change the NUMBER-OF-BALLS slider.  What is the best way to produce a standard binomial distribution (or approximate a bell curve)?

Set a CHANCE-OF-BOUNCING-RIGHT then try to predict the resulting stacks of balls.  How would you calculate the mean and variances of a given stack for a given setting?


EXTENDING THE MODEL
-------------------
Modify the program to allow independent adjustment of each peg, so that they can adjust their own orientation, rather than having all the pegs synchronized.

Change the shape of the board.  Maybe flip the triangle upside down.  How does this effect how the balls get distributed?

In addition to changing the shape of the board, change the direction balls can go.  Maybe allow balls to go in all directions.

Make it so you can select a specific peg.  If a ball bounces off that peg, stop the ball.  Keep track of how many balls are stopped.  What specific insight does this provide about the independent trials process and ball distributions?.


RELATED MODELS
--------------
Binomial Rabbits


NETLOGO FEATURES
----------------
SHADE-PATH? enables balls to increment the shade of the path they travel.  You can easily modify this function to allow different shades.  You might want to try setting a tolerance, so when a specific path is traveled a certain number of times it turns a different color.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2001).  NetLogo Galton Box model. http://ccl.northwestern.edu/netlogo/models/GaltonBox. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

circle
false
0
Circle -7566196 true true 35 35 230

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

bee
true
0
Polygon -256 true false 151 152 137 77 105 67 89 67 66 74 48 85 36 100 24 116 14 134 0 151 15 167 22 182 40 206 58 220 82 226 105 226 134 222
Polygon -16777216 true false 151 150 149 128 149 114 155 98 178 80 197 80 217 81 233 95 242 117 246 141 247 151 245 177 234 195 218 207 206 211 184 211 161 204 151 189 148 171
Polygon -7566196 true true 246 151 241 119 240 96 250 81 261 78 275 87 282 103 277 115 287 121 299 150 286 180 277 189 283 197 281 210 270 222 256 222 243 212 242 192
Polygon -16777216 true false 115 70 129 74 128 223 114 224
Polygon -16777216 true false 89 67 74 71 74 224 89 225 89 67
Polygon -16777216 true false 43 91 31 106 31 195 45 211
Line -1 false 200 144 213 70
Line -1 false 213 70 213 45
Line -1 false 214 45 203 26
Line -1 false 204 26 185 22
Line -1 false 185 22 170 25
Line -1 false 169 26 159 37
Line -1 false 159 37 156 55
Line -1 false 157 55 199 143
Line -1 false 200 141 162 227
Line -1 false 162 227 163 241
Line -1 false 163 241 171 249
Line -1 false 171 249 190 254
Line -1 false 192 253 203 248
Line -1 false 205 249 218 235
Line -1 false 218 235 200 144

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 46 130
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150
Polygon -7566196 true true 270 14
Polygon -7566196 true true 276 21

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

@#$#@#$#@
NetLogo 1.2beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
